/*
    v0.01 - original
    v0.02 - removed KEY REMAPPING
    v0.03 - changed patch.  The original would also disable the now/next on some channels.
    v0.04 - fixed for firmware versions v12.04,v12.05 (patch addresses were swapped with each other)
          - patch is now always applied on startup, can be removed via menu
          - menu displays state
    v0.05 - update for firmware v12.25
          - now uses exTap library v0.02

    a tap to disable/enable mheg
    this merely prevents the software from processing the dsmcc stream
    by zapping the subrouting call.
    Note, since you can enable/disable it on the fly; you can play tricks like:
        enable mheg
        tune to a chennel
        wait a bit to allow the cache to fill up
        disable mheg
        viewing the (now static) pages
    The cache will be deleted when you change channels.

    Use the 'Recall' key to bring up a menu (exit cancels, recall propagates to next tap/firmware).

*/

#include <tap.h>
#include "exTap.h"

//Every TAP application needs unique ID and TAP allocations that have same ID cant execute at same time.
//ID consists of DWORD( 4byte ) and the MSB bit should be set by 1.
//The following is how to use in source codes.
#define ID_USERTAP 0xbdb00105
TAP_ID( ID_USERTAP );

//Every TAP application has to include Program name, Author name, and Program description as following
//manners in source code.
TAP_PROGRAM_NAME("mheg_off");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("A TAP to disable/enable MHEG");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
#define ptrfix(p) (p + _tap_startAddr)

typedef struct {
    int  id;
    char str[0x20];
}type_item;

typedef struct {
    int  id;
    int  visible;
    TYPE_Window win;
    int  rgn;
    char *fixme_titlep;
    type_item item[0x10];
}type_menu;

//----------------------------------------------------------------------------
char menu_title[0x20] = "MHEG is ON";

type_menu menu = {
    0,
    0,
    {0},
    0,
    menu_title,
    {
        {3, "xxx",          },
        {0, "Disable MHEG", },
        {1, "Enable MHEG",  },
        {2, "Exit Tap",     },
    },
};


int TAP_Main(void);
dword TAP_EventHandler(word event, dword param1, dword param2);
int menu_handler(type_menu *menu, dword param1);
int key_handler(dword param1);
void ShowMenu(type_menu *menu);
void DeleteMenu(type_menu *menu);

//----------------------------------------------------------------------------
int TAP_Main(void)
{
    int ret;
    word SysID = *(word*)pSysID;

    TAP_Print("\nopening %s",__tap_program_name__);

    //determine support for different firmwares
    ret = exTAP_Init();
    if (ret) {
        return 0;
    }

    //apply patch
    ret = exTAP_MHEG_Disable(1);      //0=mheg on, 1=mheg off
    if (ret == 0) strcpy(menu_title, "MHEG is OFF");

    menu.visible = 0;
    return 1;                 //0=application mode, 1=tsr mode
}

//----------------------------------------------------------------------------
dword TAP_EventHandler(word event, dword param1, dword param2)
{
    int ret;
    dword state, subState;

    switch (event) {
        case EVT_IDLE:
            break;

        case EVT_KEY:
//TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);

            TAP_GetState(&state, &subState);
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                if (menu.visible) { // handling keys when options window is shown
                    param1 = menu_handler(&menu, param1);
                }
                else {
                    param1 = key_handler(param1);
                }
            }
	        break;
    }
    return param1;
}

int menu_handler(type_menu *menu, dword param1)
{
    int selection;
    int ret;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
			switch (selection) {
				case 0:
                    ret = exTAP_MHEG_Disable(1);      //0=mheg on, 1=mheg off
                    if (ret == 0) strcpy(menu_title, "MHEG is OFF");
				    return 0;
				case 1:
                    ret = exTAP_MHEG_Disable(0);      //0=mheg on, 1=mheg off
                    if (ret == 0) strcpy(menu_title, "MHEG is ON");
				    return 0;
				case 2:
                    ret = exTAP_MHEG_Disable(0);      //0=mheg on, 1=mheg off
                    if (ret == 0) strcpy(menu_title, "MHEG is ON");
                    TAP_Print("\nclosing %s",__tap_program_name__);
				    TAP_Exit();
				    return 0;
			}
            break;
        case RKEY_Recall:
			DeleteMenu(menu);
            return param1; // cascades key to other TAP's
        case RKEY_Exit:
            DeleteMenu(menu);
            return 0; // exit without Cascading
        default:
	        TAP_Win_Action(&menu->win, param1); // send all other key's to menu-Win
    		return 0;
    }
    return param1;
}


int key_handler(dword param1)
{
    switch(param1) {
        case RKEY_Recall:                 //menu
            ShowMenu(&menu);
            return 0;
    }
    return param1;
}

//----------------------------------------------------------------------------
void ShowMenu(type_menu *menu)
{
    int i;

	if (!menu->visible) {
	    menu->rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);
	    TAP_Win_SetDefaultColor(&menu->win);
	    TAP_Win_Create(&menu->win, menu->rgn, 250, 210, 200, 80, FALSE, FALSE);
        TAP_Win_SetTitle(&menu->win, ptrfix(menu->fixme_titlep), 0, FNT_Size_1622);
	    for (i=1; i<=menu->item[0].id; i++) {
    	    TAP_Win_AddItem(&menu->win, menu->item[i].str);
	    }
	    TAP_Win_SetSelection(&menu->win, 0);
	    menu->visible = 1;
	}
}

void DeleteMenu(type_menu *menu)
{
	if (menu->visible) {
	    TAP_Win_Delete(&menu->win);
    	TAP_Osd_Delete(menu->rgn);
	    menu->visible = 0;
    }
}

//----------------------------------------------------------------------------
